#include <Wire.h> 
#include <LiquidCrystal_I2C.h>
#define sensorPin 0    // Verbunden mit LM35 Ausgang
#define DELAY1 10      // Kurze Wartezeit beim Messen
#define DELAY2 500     // Kurze Wartezeit beim Anzeigen
#define motorPin 9     // Lüfter-Pin
#define threshold 27   // Schalt-Temperatur für Lüfter (25 Grad Celsius)
#define hysterese 0.5  // Hysterese-Wert (0,5 Grad Celsius)
const int cycles = 20; // Anzahl der Messungen
LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE); // Adresse auf 0x27

void setup() {                
  pinMode(motorPin, OUTPUT);
  lcd.begin(20,4); // LCD initialisieren
  lcd.backlight(); // Hintergrundbeleuchtung aktivieren
}

void loop() {
  float resultTemp = 0.0;
  for(int i = 0; i < cycles; i++) {
    int analogValue = analogRead(sensorPin);
    float temperature = (5.0 * 100.0 * analogValue) / 1024;
    resultTemp += temperature; // Aufsummieren der Messwerte
    delay(DELAY1);
  }
  resultTemp /= cycles;       // Berechnung des Durchschnitts
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("**Lueftersteuerung**");
  lcd.setCursor(0,1);
  lcd.print("Temperatur: ");
  lcd.print(resultTemp);
  lcd.write(0xD0 + 15);       // Grad-Zeichen
  lcd.print("C");
  lcd.setCursor(0,2);
  lcd.print("Motor: ");
  if(resultTemp > (threshold + hysterese))
    digitalWrite(motorPin, HIGH);
  else if(resultTemp < (threshold - hysterese))
    digitalWrite(motorPin, LOW);
  lcd.print(digitalRead(motorPin) == HIGH?"an":"aus");
  delay(DELAY2);
}

