#include "SPI.h"
#include "Adafruit_GFX.h"
#include "Adafruit_ILI9341.h"
#include <Wire.h>        // Use I2C library
#define CMPS11Addr  0x60 // Addresse des CMPS11
#define ANGLE8BIT_REG 1  // Register für 8-Bit Kompasswert

#define TFT_DC 9
#define TFT_CS 10

// RGB-Colors
#define WHITE   tft.color565(255, 255, 255) // ILI9341_WHITE
#define BLACK   tft.color565(  0,   0,   0) // ILI9341_BLACK
#define RED     tft.color565(255,   0,   0) // ILI9341_RED
#define GREEN   tft.color565(  0, 255,   0) // ILI9341_GREEN
#define BLUE    tft.color565(  0,   0, 255) // ILI9341_BLUE
#define YELLOW  tft.color565(255, 255,   0)
#define CYAN    tft.color565(  0, 255, 255)
#define MAGENTA tft.color565(255,   0, 255)

Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);

int xB = tft.width()/2, yB = 150, rB = 70;
int xP = 50,  yP = 260, rP = 40;
int xR = 190, yR = 260, rR = 40;

void setup() {
  Wire.begin();       // Init I2C
  tft.begin();        // TFT initialisieren
  tft.fillScreen(BLACK); // Hintergrund löschen
  tft.setTextSize(1); // Textgröße festlegen  
  tft.drawCircle(xB, yB, rB, RED);   // Bearing
  tft.drawCircle(xP, yP, rP, GREEN); // Pitch
  tft.drawCircle(xR, yR, rR, CYAN);  // Roll
  tft.setCursor(tft.width()/2 - 20, 225);
  tft.setTextColor(RED);
  tft.print("Bearing");
  tft.setCursor(35, 305);
  tft.setTextColor(GREEN);
  tft.print("Pitch");
  tft.setCursor(180, 305);
  tft.setTextColor(CYAN);
  tft.print("Roll");
}

void loop() {
  // byteHigh, byteLow für Bearing und Kompasswert 8-Bit
  byte byteHigh, byteLow, angle8Bit ; 
  char pitch, roll;       // Pitch und Roll
  int bearing;            // Bearing
  Wire.beginTransmission(CMPS11Addr); // Kommunikation mit CMPS11
  Wire.write(ANGLE8BIT_REG);    // Start Register (1)
  Wire.endTransmission();
  Wire.requestFrom(CMPS11Addr , 5); // Abfrage von 5 Bytes vom CMPS11
  while (Wire.available() < 5); // Warten, bis 5 Bytes verfügbar
  angle8Bit = Wire.read();      // Kompasswert 8-Bit lesen
  byteHigh = Wire.read();       // High-Byte für Bearing lesen
  byteLow = Wire.read();        // Low-Byte für Bearing lesen
  pitch = Wire.read();          // Byte für Pitch lesen
  roll = Wire.read();           // Byte für Roll lesen
  bearing = ((byteHigh << 8) + byteLow) / 10; // Bearing berechnen
  displayData(bearing, pitch, roll);          // Anzeige aufrufen
}

void displayData(int bearing, int pitch, int roll) {
  int xPosB, yPosB, xPosP, yPosP; 
  int xPosR1, yPosR1, xPosR2, yPosR2;
  // Textgröße festlegen
  tft.setTextSize(3);
  // Anzeige der Werte
  tft.setCursor(0, 0);
  tft.setTextColor(RED);
  tft.print("Bearing: "); tft.println(bearing);
  tft.setTextColor(GREEN);
  tft.print("Pitch  : "); tft.println(pitch);
  tft.setTextColor(CYAN);
  tft.print("Roll   : "); tft.println(roll);
  // Update der Grafik
  // Bearing
  xPosB = xB + (rB - 5) * cos(PI/180*(bearing - 90));
  yPosB = yB + (rB - 5) * sin(PI/180*(bearing - 90));
  tft.drawLine(xB, yB, xPosB, yPosB, YELLOW);
  // Pitch
  xPosP = xP + (rP - 5) * cos(PI/180*(pitch - 180));
  yPosP = yP + (rP - 5) * sin(PI/180*(pitch - 180));
  tft.drawLine(xP, yP, xPosP, yPosP, YELLOW);
  // Roll
  xPosR1 = xR + (rR - 5) * cos(PI/180*roll);
  yPosR1 = yR + (rR - 5) * sin(PI/180*roll);
  xPosR2 = xR + (rR - 5) * cos(PI/180*roll + PI);
  yPosR2 = yR + (rR - 5) * sin(PI/180*roll + PI);
  tft.drawLine(xR, yR, xPosR1, yPosR1, YELLOW);
  tft.drawLine(xR, yR, xPosR2, yPosR2, YELLOW);
  delay(1000); // Kurze Pause
  // Löschen der Anzeige
  tft.setCursor(0, 0);
  tft.setTextColor(BLACK);
  tft.print("Bearing: "); tft.println(bearing);
  tft.print("Pitch  : "); tft.println(pitch);
  tft.print("Roll   : "); tft.println(roll);
  tft.drawLine(xB, yB, xPosB, yPosB, BLACK);
  tft.drawLine(xP, yP, xPosP, yPosP, BLACK);
  tft.drawLine(xR, yR, xPosR1, yPosR1, BLACK);
  tft.drawLine(xR, yR, xPosR2, yPosR2, BLACK);
}
