#include "SPI.h"
#include "Adafruit_GFX.h"
#include "Adafruit_ILI9341.h"
#include <Wire.h>        // Use I2C library
#define CMPS11Addr  0x60 // Addresse des CMPS11
#define ANGLE8BIT_REG 1  // Register für 8-Bit Kompasswert

#define TFT_DC 9
#define TFT_CS 10

// RGB-Colors
#define WHITE   tft.color565(255, 255, 255) 
#define BLACK   tft.color565(  0,   0,   0) 
#define RED     tft.color565(255,   0,   0) 
#define GREEN   tft.color565(  0, 255,   0) 
#define BLUE    tft.color565(  0,   0, 255)
#define YELLOW  tft.color565(255, 255,   0)
#define CYAN    tft.color565(  0, 255, 255)
#define MAGENTA tft.color565(255,   0, 255)

Adafruit_ILI9341 tft = Adafruit_ILI9341(TFT_CS, TFT_DC);

void setup() {
  Wire.begin(); // Connect to I2C
  tft.begin();  // TFT initialisieren
  tft.fillScreen(BLACK); // Hintergrund löschen
  tft.setTextSize(3);    // Textgröße festlegen
}

void loop() {
  // byteHigh, byteLow für Bearing und Kompasswert 8-Bit
  byte byteHigh, byteLow, angle8Bit; 
  char pitch, roll;       // Pitch und Roll
  int bearing;            // Bearing

  Wire.beginTransmission(CMPS11Addr); // Kommunikation mit CMPS11
  Wire.write(ANGLE8BIT_REG);    // Start Register (1)
  Wire.endTransmission();
  Wire.requestFrom(CMPS11Addr , 5); // Abfrage von 5 Bytes vom CMPS11
  while (Wire.available() < 5); // Warten, bis 5 Bytes verfügbar
  angle8Bit = Wire.read();      // Kompasswert 8-Bit lesen
  byteHigh = Wire.read();       // High-Byte für Bearing lesen
  byteLow = Wire.read();        // Low-Byte für Bearing lesen
  pitch = Wire.read();          // Byte für Pitch lesen
  roll = Wire.read();           // Byte für Roll lesen
  bearing = ((byteHigh << 8) + byteLow) / 10; // Bearing berechnen
  displayData(bearing, pitch, roll);          // Anzeige aufrufen
}

void displayData(int bearing, int pitch, int roll) {
  // Anzeige der Werte
  tft.setCursor(0, 0);
  tft.setTextColor(RED);
  tft.print("Bearing: "); tft.println(bearing);
  tft.setTextColor(GREEN);
  tft.print("Pitch  : "); tft.println(pitch);
  tft.setTextColor(CYAN);
  tft.print("Roll   : "); tft.println(roll);
  delay(1000); // Kurze Pause
  // Löschen der Anzeige
  tft.setCursor(0, 0);
  tft.setTextColor(BLACK);
  tft.print("Bearing: "); tft.println(bearing);
  tft.print("Pitch  : "); tft.println(pitch);
  tft.print("Roll   : "); tft.println(roll);
}


