#include "ArduBotMotor.h"
#include "KeyDef.h"
#define BUZZER_PIN 8    // Buzzer-Pin
#define STEPS 10        // Anzahl der erlaubten Schritte
#define TONE_CMD   200  // Ton for CMD
#define TONE_DIGIT 400  // Ton for Digit

#define BIGTRAK_MOVE_DURATION  500  // Parameter vorwärts/rückwärts
#define BIGTRAK_TURN_DURATION 70    // Parameter drehen

byte keyValue;             // Speichert keyValue
byte threshold;            // Threshold für Tastendruck
unsigned int debounceTime; // Debounce-Wert
long lastTimeValue;        // Wird für Debouncing benötigt
byte mode, digCount;       // Speicher-Mode, Tastendruck-Zähler
String actionValue;        // Speichert action-value
String actionStep[STEPS];  // Array für Aktionen
int stepCounter;           // Zählt Aktionen

ArduBotMotor abm = ArduBotMotor(2, 3, 4, 5, 10, 11); // Motorinstanz erzeugen

void setup() {
  Serial.begin(9600); // Init Serial
  threshold = 3;      // Threshold für Tasten
  debounceTime = 500; // Debounce-Wert
  abm.pwm(M1, 255);   // PWM-Wert
  abm.pwm(M2, 255);   // PWM-Wert
  clearActionSteps(STEPS);
}

void loop() {
  char key = readKey();
    if(key != MyKEY_NOT_PRESSED) {
      if(key == MyKEY_GO) {
        if((mode == 1) && (actionValue.length() > 2) && (stepCounter < STEPS)) {
          Serial.println("Schritt zum Array hinzugefügt");
          actionStep[stepCounter] = actionValue; // Speichert Aktion in actionStep-Array
          digCount = 0;
          actionValue = "A";
          actionValue += key;
          playTone(TONE_CMD, 150);
          Serial.println(actionValue);
          delay(500);
          playMelody();
          action();
        }
      }              
      if((key == MyKEY_DOWN) || (key == MyKEY_LEFT) || (key == MyKEY_UP) || (key == MyKEY_RIGHT) || (key == MyKEY_PAUSE) || (key == MyKEY_FIRE)) {
        Serial.println("CMD");
        if(mode == 0) {
          actionValue += key;
          mode = 1;
          playTone(TONE_CMD, 150);
          Serial.println("ActionValue1: " + actionValue);
        }       
        if((mode == 1) && (actionValue.length() > 2) && (stepCounter < STEPS)) {
          Serial.println("Schritt zum Array hinzugefügt");
          actionStep[stepCounter++] = actionValue; // Speichert Aktion in actionStep-Array
          digCount = 0;
          actionValue = "A";
          actionValue += key;
          playTone(TONE_CMD, 150);
          Serial.println(actionValue);
        }
      } else
      if((mode == 1) && (digCount < 3)){
          if((key == MyKEY_0) || (key == MyKEY_1) || (key == MyKEY_2) || (key == MyKEY_3) || (key == MyKEY_4) || (key == MyKEY_5) || (key == MyKEY_6) || (key == MyKEY_7) || (key == MyKEY_8) || (key == MyKEY_9)) {
                Serial.println("Nummer");
                actionValue += key;
                digCount++; // Inkrementiert digit-count
                playTone(TONE_DIGIT, 150);
             }
        Serial.println("ActionValue3: " + actionValue);
        }  
    printSteps();
    }
}

void printSteps(void) {
  Serial.println("=============");
  for(int i = 0; i < STEPS; i++) {
    Serial.println(actionStep[i]);
  }
}

char readKey(void) {
    keyValue = analogRead(A0);
    char key = MyKEY_NOT_PRESSED;
    if((keyValue > 0) && (millis() - lastTimeValue >= debounceTime)) {
      if((keyValue > (231 - threshold)) && (keyValue < (231  + threshold))) key = MyKEY_GO;
      if((keyValue > (220 - threshold)) && (keyValue < (220  + threshold))) key = MyKEY_CE;
      if((keyValue > (210 - threshold)) && (keyValue < (210  + threshold))) key = MyKEY_CM;
      if((keyValue > (199 - threshold)) && (keyValue < (199  + threshold))) key = MyKEY_FIRE;

      if((keyValue > (188 - threshold)) && (keyValue < (188  + threshold))) key = MyKEY_PAUSE;
      if((keyValue > (177 - threshold)) && (keyValue < (177  + threshold))) key = MyKEY_RIGHT;
      if((keyValue > (166 - threshold)) && (keyValue < (166  + threshold))) key = MyKEY_UP;
      if((keyValue > (155 - threshold)) && (keyValue < (155  + threshold))) key = MyKEY_LEFT;
      if((keyValue > (143 - threshold)) && (keyValue < (143  + threshold))) key = MyKEY_DOWN;

      if((keyValue > (132 - threshold)) && (keyValue < (132  + threshold))) key = MyKEY_9;
      if((keyValue > (120 - threshold)) && (keyValue < (120  + threshold))) key = MyKEY_8;
      if((keyValue > (108 - threshold)) && (keyValue < (108  + threshold))) key = MyKEY_7;
      if((keyValue > (95  - threshold)) && (keyValue < (95   + threshold))) key = MyKEY_6;
      if((keyValue > (83  - threshold)) && (keyValue < (83   + threshold))) key = MyKEY_5;
      if((keyValue > (70  - threshold)) && (keyValue < (70   + threshold))) key = MyKEY_4;
      if((keyValue > (56  - threshold)) && (keyValue < (56   + threshold))) key = MyKEY_3;
      if((keyValue > (42  - threshold)) && (keyValue < (42   + threshold))) key = MyKEY_2;
      if((keyValue > (28  - threshold)) && (keyValue < (28   + threshold))) key = MyKEY_1;
      if((keyValue > (13  - threshold)) && (keyValue < (13   + threshold))) key = MyKEY_0;
      lastTimeValue = millis();
    }
    return key;
}

void clearActionSteps(int steps) {
  mode = 0;
  digCount = 0;
  stepCounter = 0;
  actionValue = "A";
  for(int i = 0; i < steps; i++)
    actionStep[i] = "."; // Leerer Schritt
}

void action(void) {
  Serial.println("Action");
  printSteps();
  String action;
  int actionValue;
  for(int i = 0; i < STEPS; i++) {
    if(actionStep[i] != ".") {
      action = actionStep[i].substring(1, 2);
      actionValue = actionStep[i].substring(2).toInt();
      arduTrakMove(action, actionValue);
    }
  }
  playMelody();             // Spiele Melodie
  clearActionSteps(STEPS);  // Lösche Schritte
}

void arduTrakMove(String direction, int duration) {
  // Move forward
  if(direction == "U")
    abm.move(FORWARD, BIGTRAK_MOVE_DURATION * duration);
  // Move backward
  if(direction == "D")
    abm.move(BACKWARD, BIGTRAK_MOVE_DURATION * duration);
  // Turn right
  if(direction == "R")
    abm.move(RIGHT, BIGTRAK_TURN_DURATION * duration);
  // Turn left
  if(direction == "L")
    abm.move(LEFT, BIGTRAK_TURN_DURATION * duration);
  // Pause
  if(direction == "P")
    delay(1000 * duration);
  if(direction == "F") {
    for(int i = 0; i < duration; i++)
      for(int j = 1600; j > 50; j--) {
        tone(BUZZER_PIN, j, 50); delayMicroseconds(100);
      }
  }
  delay(300); // Kurze Pause
}

void playTone(int t, int duration) {
  tone(BUZZER_PIN, t, duration);
}

void playMelody(void) {
    tone(BUZZER_PIN, 330, 200); delay(200);
    tone(BUZZER_PIN, 349, 200); delay(200);
    tone(BUZZER_PIN, 392, 200); delay(200);
    tone(BUZZER_PIN, 349, 200); delay(200);
    tone(BUZZER_PIN, 247, 400); delay(250); 
    tone(BUZZER_PIN, 247, 400); delay(250);
}
